Make a read splice from a DAX file go directly to copy_splice_read() to do
the reading as filemap_splice_read() is unlikely to find any pagecache to
splice.

I think this affects only erofs, Ext2, Ext4, fuse and XFS.

Signed-off-by: David Howells <dhowe...@redhat.com>
Reviewed-by: Christoph Hellwig <h...@lst.de>
Reviewed-by: Christian Brauner <brau...@kernel.org>
Reviewed-by: Theodore Ts'o <ty...@mit.edu>
Reviewed-by: Gao Xiang <hsiang...@linux.alibaba.com>
cc: Al Viro <v...@zeniv.linux.org.uk>
cc: Jens Axboe <ax...@kernel.dk>
cc: linux-erofs@lists.ozlabs.org
cc: linux-e...@vger.kernel.org
cc: linux-...@vger.kernel.org
cc: linux-fsde...@vger.kernel.org
cc: linux-bl...@vger.kernel.org
cc: linux...@kvack.org
---

Notes:
    ver #21)
     - Don't need #ifdef CONFIG_FS_DAX as IS_DAX() is false if !CONFIG_FS_DAX.
     - Needs to be in vfs_splice_read(), not generic_file_splice_read().

 fs/splice.c | 6 +++---
 1 file changed, 3 insertions(+), 3 deletions(-)

diff --git a/fs/splice.c b/fs/splice.c
index 76126b1aafcb..8268248df3a9 100644
--- a/fs/splice.c
+++ b/fs/splice.c
@@ -908,10 +908,10 @@ long vfs_splice_read(struct file *in, loff_t *ppos,
        if (unlikely(!in->f_op->splice_read))
                return warn_unsupported(in, "read");
        /*
         * O_DIRECT doesn't deal with the pagecache, so we allocate a buffer,
         * copy into it and splice that into the pipe.
         */
-       if ((in->f_flags & O_DIRECT))
+       if ((in->f_flags & O_DIRECT) || IS_DAX(in->f_mapping->host))
                return copy_splice_read(in, ppos, pipe, len, flags);
        return in->f_op->splice_read(in, ppos, pipe, len, flags);
 }
@@ -908,10 +908,10 @@ long vfs_splice_read(struct file *in, loff_t *ppos,
                return warn_unsupported(in, "read");
        /*
-        * O_DIRECT doesn't deal with the pagecache, so we allocate a buffer,
-        * copy into it and splice that into the pipe.
+        * O_DIRECT and DAX don't deal with the pagecache, so we allocate a
+        * buffer, copy into it and splice that into the pipe.
         */

Reply via email to